// Author: Alberto González Palomo http://sentido-labs.com
// ©2017 Alberto González Palomo http://sentido-labs.com
// Created: 2017-02-22 02:22
var model = { thingsSaid: "Hello, world!", onOff: true };

var nexus = dom.Nexus.new(model);

// Define a couple of UI components to encapsulate text fields and checkboxes:
var InputField = {
    new: function InputField(property, nexus, attributes) {
        var view = dom(['input', attributes]);
        return nexus.bind(this, property, 'input', view);
    },
    get: function get(view) { return view.value; },
    set: function set(view, value) { view.value = value; }
};

var CheckBox = {
    new: function CheckBox(property, nexus, attributes) {
        attributes.type = 'checkbox';
        var view = dom(["input", attributes]);
        return nexus.bind(this, property, 'change', view);
    },
    get: function get(view) { return view.checked; },
    set: function set(view, value) { view.checked = value; }
};

// Set some attributes in the <body> tag:
dom([ document.body, { "style": "font-family:sans-serif" } ])

// Styles can be attributes as above, or CSS rules in a <style> element:
dom([
    document.body, // Insert rest as attributes/children of document.body.
    ["style", { "type":"text/css" }, "section { margin:1em; }"]
]);

// Here we get the document title from the <title> tag, and replace the
// contents of the <h1> tag inside the <header> with it:
dom([ dom.clear(dom.$('header > h1')), document.title ]);

dom([
    dom.id("main"),
    ["section",
     ["p", "All input elements are bound through the model object which you can see at the bottom of this page."],
     InputField.new("thingsSaid", nexus, {
         "autofocus": true,
         "placeholder": "Please, tell me something."
     }),
     ["label", CheckBox.new("onOff", nexus, {}), " on/off, just a checkbox example"]
    ],

    ["section", ["p", "Just some paragraph in between."] ],
    ["section",
     InputField.new("thingsSaid", nexus, { "placeholder": "Please?" })
    ],
    ["section", ["p", "Another one, there is no limit!"] ],
    ["section",
     InputField.new("thingsSaid", nexus, { "placeholder": "Could you?" })
    ],
    ["section",
     ["p", "Another synchronized check box:"],
     ["label", CheckBox.new("onOff", nexus, {}), " this checkbox is synchronized with the on/off one"],
    ]
]);

// Add a timestamp:
dom([
    dom.$("footer"),
    "Last viewed: ", (new Date()).toISOString().replace(/T.*/, '')
]);

// Watch changes in model.thingsSaid and copy the value to #old-paper:
nexus.addListener('thingsSaid', function (model, property, origin) {
    dom.id('old-paper').textContent = model[property];
});

// Display the whole model any time a property changes:
nexus.addListener("*", function (model, property, origin) {
    dom.id("model-src").textContent = JSON.stringify(model, null, '  ');
});
