// Author: Alberto González Palomo <http://sentido-labs.com>
// ©2017 Alberto González Palomo <http://sentido-labs.com>
// Created: 2017-03-03 22:11
// http://bulma.io/documentation/components/card/
var Bulma = (function () {
    // http://bulma.io/documentation/components/card/

    var API = {};

    var transitionDuration = '0.5s';

    function update(view, nexus, property, allValues)
    {
        var newState = nexus.model[property];
        allValues.forEach(function (state) {
            if (state && state !== newState) view.classList.remove(state);
        });
        if (newState) view.classList.add(newState);
    }

    if (!window.requestAnimationFrame) {
        console.log('Simulating requestAnimationFrame with setTimeout.');
        window.requestAnimationFrame = function(f) {
            return setTimeout(f, 16);
        };
    }

    function waitFrames(frames, callback)
    {
        requestAnimationFrame(function () {
            if (frames > 0) waitFrames(frames - 1, callback);
            else            callback();
        });
    }

    API.Button = {
        new: function Button(property, transitions, nexus) {
            return this.use(dom(["button", {"name":property} ]),
                            transitions, nexus);
        },
        use: function Button(view, transitions, nexus) {
            view.classList.add('button');
            var property = view.name;
            if (!property || !nexus) return view;

            var allValues = [];
            nexus.addListener(property, function (model, property, origin) {
                update(view, nexus, property, allValues);
            });
            if (!transitions) return view;

            Object.keys(transitions).forEach(function (eventName) {
                var states = transitions[eventName];
                if ('string' === typeof states) states = ["", states];
                view.addEventListener(eventName, function cycle(event) {
                    var state = nexus.model[property];
                    var index = states.indexOf(state);
                    if (index >= 0) {
                        nexus.model[property] = states[(index + 1)
                                                       % states.length];
                        nexus.changed(property);
                    }
                });
                states.forEach(function (state) {
                    if (allValues.indexOf(state) < 0) allValues.push(state);
                });
            });
            return view;
        }
    };

    API.Input = {
        new: function Input(property, nexus) {
            return this.use(dom(["input", { "name":property } ]), nexus);
        },
        use: function Input(view, nexus) {
            var property = view.name;
            view.classList.add('input');
            return nexus.bind(this, property, 'input', view);
        },
        get: function get(view) { return view.value; },
        set: function set(view, value) { view.value = value; }
    };

    API.Progress = {
        new: function Progress(property, nexus) {
            return this.use(property, dom(["progress"]));
        },
        use: function Progress(view, property, nexus) {
            view.classList.add('progress');
            if (!property || !nexus) return view;
            return nexus.bind(this, property, null, view);
        },
        get: function get(view) { return view.value; },
        set: function set(view, value) { view.value = value; }
    };

    function deleteItem(event)
    {
        var durationMs = Number(transitionDuration.replace('s', '')) * 1000;
        var item = event.target;
        while (item &&
               !item.classList.contains('notification') &&
               !item.classList.contains('message')) {
            item = item.parentNode;
        }
        // Just crash if item is not found.
        item.style.transition =
            'opacity ' + transitionDuration +
            ', margin-top ' + transitionDuration +
            ', margin-bottom ' + transitionDuration;
        var height;
        var s = item.style;
        s.opacity = '1.0';
        if (item.nextElementSibling) {
            height = item.offsetHeight;
            s.marginBottom = '0';
            s.marginBottom = (-height) + 'px';
        } else if (item.previousElementSibling) {
            var previous = item.previousElementSibling;
            height =   item.offsetTop +     item.offsetHeight
                - (previous.offsetTop + previous.offsetHeight);
            s.marginTop = '0';
            s.marginTop = (-height) + 'px';
        }
        s.opacity = '0.0';

        setTimeout(function () {
            if (!item.parentNode) return;
            var previous = item.previousElementSibling;
            if (previous && !item.nextElementSibling) {
                var transition = previous.style.transition;
                previous.style.transition = '';
                item.parentNode.removeChild(item);
                waitFrames(1, function () {
                    previous.style.transition = transition;
                });
            } else {
                item.parentNode.removeChild(item);
            }
        }, durationMs);
    }

    API.DeleteButton = {
        new: function DeleteButton(timeout) {
            var view = dom(["button.delete"])
            return this.use(view, timeout);
        },
        use: function DeleteButton(view, timeout) {
            dom([view, { "click": deleteItem } ]);
            if (timeout) setTimeout(function () { view.click(); }, timeout);
            return view;
        }
    };

    API.Notification = {
        new: function Notification(className, content, timeout) {
            var notification = dom([
                "div", { "class": "notification " + className },
                API.DeleteButton.new(timeout),
                content
            ]);
            return notification;
        }
    };

    API.Message = {
        new: function Message(template, title, content, timeout) {
            if ('string' === typeof template) {
                template = dom([
                    "article", { "class": "message " + className },
                    ["div.message-header"],
                    ["div.message-body"]
                ]);
            }
            if (title) {
                dom([dom.clear(dom.$('.message-header', template)), title]);
            }
            if (content) {
                dom([dom.clear(dom.$('.message-body', template)), content]);
            }
            API.DeleteButton.new(message, '.message-header', timeout);
            return message;
        }
    };

    API.Breadcrumb = {
        new: function Breadcrumb(property, nexus) {
            return this.use(property, dom(["nav"]));
        },
        use: function Breadcrumb(view, property, nexus) {
            view.classList.add('breadcrumb');
            view.setAttribute('aria-label', 'breadcrumbs');
            if (!dom.$('ul', view)) dom([view, ["ul"]]);
            if (!property || !nexus) return view;
            return nexus.bind(this, property, null, view);
        },
        get: function get(view) {
            var value = [];
            var i = dom.iter('li > a', view);
            var a;
            while ((a = i.iterateNext())) values.push(a.href);
            return values;
        },
        set: function set(view, value) {
            dom([dom.clear(dom.$('ul', view)),
                 value.reduce(function (a, item) {
                     a.push(["li", ["a", {"href":item}, item]]);
                     return a;
                 }, [""])
                ]);
        }
    };

    return API;
})();
