// Author: Alberto González Palomo https://sentido-labs.com
// ©2019 Alberto González Palomo https://sentido-labs.com

dom.XInclude = (function () {
    const namespaces = {
        'xi': 'http://www.w3.org/2001/XInclude'
    };

    // - getXML: function (href, callback)
    // - callback: function (err, document)
    function expandXInclude(xml, baseHref, getXML, callback, maxExpansions)
    {
        if (isNaN(maxExpansions)) maxExpansions = 1;
        var nextXInclude = dom.XPath.new(xml, namespaces)('//xi:include');
        function expand(baseHref, callback)
        {
            baseHref = baseHref.replace(/[^\/]+$/, '');
            var node = maxExpansions > 0? nextXInclude.node(xml): null;
            --maxExpansions;
            if (node) {
                var href = baseHref + node.getAttribute('href');
                getXML(href, (err, xmlToInclude) => {
                    if (err) return callback(err);
                    var ns = {};
                    var p = parseXPointer(node.getAttribute('xpointer'));
                    var xpath = dom.XPath.new(xmlToInclude, p.ns)(p.path);
                    var parent = node.parentNode;
                    var nodesToInclude = [];
                    dom.iterateAsync(xpath.iter(xmlToInclude), (n) => {
                        nodesToInclude.push(n);
                    }, null, () => {
                        nodesToInclude.forEach((nodeToInclude) => {
                            xml.adoptNode(nodeToInclude);
                            parent.insertBefore(nodeToInclude, node);
                        });
                        parent.removeChild(node);
                        expand(href, callback);
                    });
                });
            } else {
                callback(null, xml);
            }
        }
        expand(baseHref, callback);
    }

    function parseXPointer(xpointer)
    {
        var path = xpointer;
        var ns = {};
        if (path) {
            // Assumes all xmlns() appear before xpointer().
            path = path.replace(/xmlns\(([^)]*)\)/g, (m, xmlns) => {
                var i = xmlns.indexOf('=');
                ns[xmlns.substr(0, i)] = xmlns.substr(i+1);
                return '';
            }).replace(/^\s*(?:xpointer|xpath1|xpath)\((.*)\)\s*$/, '$1');
        } else {
            path = '*';
        }

        return { path:path, ns:ns };
    }

    return {
        'expand': expandXInclude
    };
})();
