// Author: Alberto González Palomo https://sentido-labs.com
// ©2018 Alberto González Palomo https://sentido-labs.com

/**
 * Patterns have the form `["tag-name", {"attr-name": "default-value", ...}]`.
 * `"tag-name"` can be the empty string `""`.
 */
dom.buildTemplate = function buildTemplate(node, allowedTags) {
    var template = null;
    if (node.nodeType === node.TEXT_NODE) {
        template = node.nodeValue.trim();
    } else {
        var nodeName =
            (node.namespaceURI? '{' + node.namespaceURI + '}': '') +
            node.localName;
        if (!allowedTags.hasOwnProperty(nodeName)) {
            console.error('FORBIDDEN TAG:', node);
        } else {
            var pattern = allowedTags[nodeName];
            template = [pattern[0]];
            var attributes = pattern[1];
            if (attributes) {
                var a = null;
                Object.keys(attributes).forEach(function (name) {
                    var value = attributes[name];// Default value.
                    if (node.hasAttribute(name)) {
                        value = node.getAttribute(name) || value;
                    }
                    if (value) {
                        if (!a) a = {};
                        a[name] = value;
                    }
                });
                if (a) template.push(a);
            }
            var child = node.firstChild;
            while (child) {
                var value = buildTemplate(child, allowedTags);
                if (value) template.push(value);
                child = child.nextSibling;
            }
        }
    }

    return template;
};
