// Author: Alberto González Palomo https://sentido-labs.com
// ©2018 Alberto González Palomo https://sentido-labs.com

// eventListeners: object with event names as keys and standard
//      DOM event listeners as values.
//      The special key '' can be used for customizing the XMLHttpRequest
//      before sending the request, which is passed to it as 'this'.
//      For instance, we can set the MIME type for "file:" requests
//      which would otherwise be "text/xml" by default:
//            '': function () {
//                if ('file:' === document.location.protocol) {
//                    this.overrideMimeType('application/json');
//                }
//            }
//      Typical values are 'progress', 'load', 'abort', 'error'.
function req(method, url, body, eventListeners)
{
    var req = new XMLHttpRequest();
    Object.keys(eventListeners).forEach(function (name) {
        if (name) req.addEventListener(name, eventListeners[name]);
        else      eventListeners[name].call(req);
    });
    req.open(method, url);
    var data, type;
    if (body) {
        data = body.string;
        type = body.type;
        if (body.json) {
            data = JSON.stringify(body.json);
            type = 'application/json';
        }
    }
    if (/^p(?:os|u)t$/i.test(method)) {
        if (type) req.setRequestHeader('Content-Type', type);
        req.send(data);
    } else {
        req.send();
    }
}
